function [Xn, Fn] = major( R, d, ftol, gradtol )
%MAJOR Majorisation algorithm for rank reduction of correlation matrices
%  Majorization algorithm for minimization of F(X)=norm(R-X*X','fro')^2 such that diag(X*X')=I with X an nxd matrix with n=size(R,1)=size(R,2). Convergence is
%  guaranteed if the matrix R and has unit diagonal. 
%  gradtol is the convergence criterion for the norm of the gradient 4*Psi*X with Psi = X*X'-R.
%  ftol is the convergence criterion for the improvement in the function
%  value F. Both criteria should be satisfied for convergence.
%  usage [Xn, Fn] = major( R, d, ftol, gradtol )

    % first obtain initial guess
    X=guess(R,d);
    
    % initial gradient, obj value
    mygrad=grad( X, R );
    normgrad=norm(mygrad,'fro');
    Fn=F(X,R);
    Fold=Fn*(1.0+ftol)+1.0; % at first iteration no Fold is known, this choice guarantees that convergence is not obtained at the first iteration
    
    % output
    n=0;
    disp(sprintf('%s\t%s\t\t%s','iter','|grad|','F(X)'));
	disp(sprintf('%d\t%e\t%e\t%s',n,normgrad,Fn));
    
    % dimension
    N=size(R,1);
    
    % eigenvectors
    eigvec = ones(d,N);

    while (normgrad>gradtol) | ( Fold/Fn-1.0 > ftol)
                
        for i=1:N

            % calculate B and its largest eigenvalue lambda
            B=X'*X-X(i,:)'*X(i,:);
            [eigvec(:,i),lambda] = powereig(eigvec(:,i),B,1e-10);
            
            % calculate the position at which the minimum of the
            % majorization function is obtained
            z=(lambda-1.0)*X(i,:)-(B*X(i,:)')'+R(i,:)*X;
            X(i,:)=z/norm(z);
        
        end
        
        % gradient, obj value
        n=n+1;
        mygrad=grad( X, R );
        normgrad=norm(mygrad,'fro');
        Fold=Fn;
        Fn=F(X,R);
        
        % output
        disp(sprintf('%d\t%e\t%e\t%s',n,normgrad,Fn));
        
    end
    
Xn=X;
